"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const fast_json_patch_1 = require("fast-json-patch");
const fs = require("fs");
const path = require("path");
const validator_1 = require("../src/validator");
describe('API', () => {
    it('should return none if data is same', () => {
        const file1 = path.resolve('resources/data.schema');
        const file2 = path.resolve('resources/data.schema');
        const obj = (0, validator_1.validateSchemaFiles)(file1, file2);
        assert.equal(obj, undefined, 'Object is defined');
    });
    it('should throw error on remove', () => {
        const file1 = path.resolve('resources/data.schema');
        const file2 = path.resolve('resources/data_v2.schema');
        assert.throws(() => {
            (0, validator_1.validateSchemaFiles)(file1, file2);
        }, Error);
    });
    it("should throw if node is added and it's required", () => {
        const schemaPath = path.resolve(`${__dirname}/../resources/data.schema`);
        const originalSchema = JSON.parse(fs.readFileSync(schemaPath, { encoding: 'utf-8' }));
        const newSchema = (0, fast_json_patch_1.deepClone)(originalSchema);
        newSchema.definitions.field = { type: 'number' };
        newSchema.required.push('field');
        assert.throws(() => (0, validator_1.validateSchemaCompatibility)(originalSchema, newSchema));
    });
    it("should return none if field is added and it's not required", () => {
        const schemaPath = path.resolve(`${__dirname}/../resources/data.schema`);
        const originalSchema = JSON.parse(fs.readFileSync(schemaPath, { encoding: 'utf-8' }));
        const newSchema = (0, fast_json_patch_1.deepClone)(originalSchema);
        newSchema.definitions.mntent.properties.field = { type: 'number' };
        assert.doesNotThrow(() => (0, validator_1.validateSchemaCompatibility)(originalSchema, newSchema));
    });
    it('should throw if field becomes required', () => {
        const schemaPath = path.resolve(`${__dirname}/../resources/data.schema`);
        const originalSchema = JSON.parse(fs.readFileSync(schemaPath, { encoding: 'utf-8' }));
        const newSchema = (0, fast_json_patch_1.deepClone)(originalSchema);
        newSchema.required = ['/'];
        assert.throws(() => (0, validator_1.validateSchemaCompatibility)(newSchema, originalSchema));
    });
    it('should return none if field becomes optional', () => {
        const schemaPath = path.resolve(`${__dirname}/../resources/data.schema`);
        const originalSchema = JSON.parse(fs.readFileSync(schemaPath, { encoding: 'utf-8' }));
        const newSchema = (0, fast_json_patch_1.deepClone)(originalSchema);
        newSchema.required = ['/'];
        assert.doesNotThrow(() => (0, validator_1.validateSchemaCompatibility)(originalSchema, newSchema));
    });
    it('should throw if field changes its type', () => {
        const schemaPath = path.resolve(`${__dirname}/../resources/data.schema`);
        const originalSchema = JSON.parse(fs.readFileSync(schemaPath, { encoding: 'utf-8' }));
        const newSchema = (0, fast_json_patch_1.deepClone)(originalSchema);
        newSchema.definitions.mntent = { type: 'number' };
        assert.throws(() => (0, validator_1.validateSchemaCompatibility)(originalSchema, newSchema));
    });
    it("should throw even if node is added and it's required under subnodes", () => {
        const schemaPath = path.resolve(`${__dirname}/../resources/data.schema`);
        const originalSchema = JSON.parse(fs.readFileSync(schemaPath, { encoding: 'utf-8' }));
        const newSchema = (0, fast_json_patch_1.deepClone)(originalSchema);
        newSchema.definitions.mntent.properties.field = { type: 'number' };
        newSchema.definitions.mntent.required.push('field');
        assert.throws(() => (0, validator_1.validateSchemaCompatibility)(originalSchema, newSchema));
    });
});
//# sourceMappingURL=validate.spec.js.map